@@ -30,7 +30,9 @@ class DeclarativeOauth2Authenticator(AbstractOauth2Authenticator, DeclarativeAut
         config (Mapping[str, Any]): The user-provided configuration as specified by the source's spec
         scopes (Optional[List[str]]): The scopes to request
         token_expiry_date (Optional[Union[InterpolatedString, str]]): The access token expiration date
+        token_expiry_date_format str: format of the datetime; provide it if expires_in is returned in datetime instead of seconds
         refresh_request_body (Optional[Mapping[str, Any]]): The request body to send in the refresh request
+        grant_type: The grant_type to request for access_token
     """
 
     token_refresh_endpoint: Union[InterpolatedString, str]
@@ -42,9 +44,11 @@ class DeclarativeOauth2Authenticator(AbstractOauth2Authenticator, DeclarativeAut
     scopes: Optional[List[str]] = None
     token_expiry_date: Optional[Union[InterpolatedString, str]] = None
     _token_expiry_date: pendulum.DateTime = field(init=False, repr=False, default=None)
+    token_expiry_date_format: str = None
     access_token_name: Union[InterpolatedString, str] = "access_token"
     expires_in_name: Union[InterpolatedString, str] = "expires_in"
     refresh_request_body: Optional[Mapping[str, Any]] = None
+    grant_type: Union[InterpolatedString, str] = "refresh_token"
 
     def __post_init__(self, options: Mapping[str, Any]):
         self.token_refresh_endpoint = InterpolatedString.create(self.token_refresh_endpoint, options=options)
@@ -53,6 +57,7 @@ class DeclarativeOauth2Authenticator(AbstractOauth2Authenticator, DeclarativeAut
         self.refresh_token = InterpolatedString.create(self.refresh_token, options=options)
         self.access_token_name = InterpolatedString.create(self.access_token_name, options=options)
         self.expires_in_name = InterpolatedString.create(self.expires_in_name, options=options)
+        self.grant_type = InterpolatedString.create(self.grant_type, options=options)
         self._refresh_request_body = InterpolatedMapping(self.refresh_request_body or {}, options=options)
         self._token_expiry_date = (
             pendulum.parse(InterpolatedString.create(self.token_expiry_date, options=options).eval(self.config))
@@ -82,14 +87,20 @@ class DeclarativeOauth2Authenticator(AbstractOauth2Authenticator, DeclarativeAut
     def get_expires_in_name(self) -> InterpolatedString:
         return self.expires_in_name.eval(self.config)
 
+    def get_grant_type(self) -> InterpolatedString:
+        return self.grant_type.eval(self.config)
+
     def get_refresh_request_body(self) -> Mapping[str, Any]:
         return self._refresh_request_body.eval(self.config)
 
     def get_token_expiry_date(self) -> pendulum.DateTime:
         return self._token_expiry_date
 
-    def set_token_expiry_date(self, value: pendulum.DateTime):
-        self._token_expiry_date = value
+    def set_token_expiry_date(self, initial_time: pendulum.DateTime, value: Union[str, int]):
+        if self.token_expiry_date_format:
+            self._token_expiry_date = pendulum.from_format(value, self.token_expiry_date_format)
+        else:
+            self._token_expiry_date = initial_time.add(seconds=value)
 
     @property
     def access_token(self) -> str:
