@@ -2,15 +2,16 @@
 # Copyright (c) 2022 Airbyte, Inc., all rights reserved.
 #
 
-from abc import ABC, abstractmethod
+from abc import abstractmethod
 from dataclasses import dataclass
 from typing import Optional
 
 from airbyte_cdk.sources.declarative.types import Config, Record, StreamSlice, StreamState
+from dataclasses_jsonschema import JsonSchemaMixin
 
 
 @dataclass
-class RecordTransformation(ABC):
+class RecordTransformation(JsonSchemaMixin):
     """
     Implementations of this class define transformations that can be applied to records of a stream.
     """
