@@ -13,6 +13,7 @@ from airbyte_cdk.sources.declarative.requesters.request_option import RequestOpt
 from airbyte_cdk.sources.declarative.stream_slicers.datetime_stream_slicer import DatetimeStreamSlicer
 
 datetime_format = "%Y-%m-%dT%H:%M:%S.%f%z"
+cursor_granularity = "PT0.000001S"
 FAKE_NOW = datetime.datetime(2022, 1, 1, tzinfo=datetime.timezone.utc)
 
 config = {"start_date": "2021-01-01T00:00:00.000000+0000", "start_date_ymd": "2021-01-01"}
@@ -33,27 +34,28 @@ def mock_datetime_now(monkeypatch):
 
 
 @pytest.mark.parametrize(
-    "test_name, stream_state, start, end, step, cursor_field, lookback_window, datetime_format, expected_slices",
+    "test_name, stream_state, start, end, step, cursor_field, lookback_window, datetime_format, cursor_granularity, expected_slices",
     [
         (
             "test_1_day",
             None,
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            "1d",
+            "P1D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-01T00:00:00.000000+0000"},
-                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T00:00:00.000000+0000"},
-                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T00:00:00.000000+0000"},
-                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T00:00:00.000000+0000"},
-                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
-                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T00:00:00.000000+0000"},
-                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T00:00:00.000000+0000"},
-                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T00:00:00.000000+0000"},
-                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T00:00:00.000000+0000"},
+                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-01T23:59:59.999999+0000"},
+                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T23:59:59.999999+0000"},
+                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T23:59:59.999999+0000"},
+                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T23:59:59.999999+0000"},
+                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T23:59:59.999999+0000"},
+                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T23:59:59.999999+0000"},
+                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T23:59:59.999999+0000"},
+                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T23:59:59.999999+0000"},
+                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T23:59:59.999999+0000"},
                 {"start_time": "2021-01-10T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
             ],
         ),
@@ -62,15 +64,16 @@ def mock_datetime_now(monkeypatch):
             None,
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            "2d",
+            "P2D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-02T00:00:00.000000+0000"},
-                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-04T00:00:00.000000+0000"},
-                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-06T00:00:00.000000+0000"},
-                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-08T00:00:00.000000+0000"},
+                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-02T23:59:59.999999+0000"},
+                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-04T23:59:59.999999+0000"},
+                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-06T23:59:59.999999+0000"},
+                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-08T23:59:59.999999+0000"},
                 {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
             ],
         ),
@@ -79,16 +82,17 @@ def mock_datetime_now(monkeypatch):
             None,
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2021-02-10T00:00:00.000000+0000", options={}),
-            "1w",
+            "P1W",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-07T00:00:00.000000+0000"},
-                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-14T00:00:00.000000+0000"},
-                {"start_time": "2021-01-15T00:00:00.000000+0000", "end_time": "2021-01-21T00:00:00.000000+0000"},
-                {"start_time": "2021-01-22T00:00:00.000000+0000", "end_time": "2021-01-28T00:00:00.000000+0000"},
-                {"start_time": "2021-01-29T00:00:00.000000+0000", "end_time": "2021-02-04T00:00:00.000000+0000"},
+                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-07T23:59:59.999999+0000"},
+                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-14T23:59:59.999999+0000"},
+                {"start_time": "2021-01-15T00:00:00.000000+0000", "end_time": "2021-01-21T23:59:59.999999+0000"},
+                {"start_time": "2021-01-22T00:00:00.000000+0000", "end_time": "2021-01-28T23:59:59.999999+0000"},
+                {"start_time": "2021-01-29T00:00:00.000000+0000", "end_time": "2021-02-04T23:59:59.999999+0000"},
                 {"start_time": "2021-02-05T00:00:00.000000+0000", "end_time": "2021-02-10T00:00:00.000000+0000"},
             ],
         ),
@@ -97,16 +101,17 @@ def mock_datetime_now(monkeypatch):
             None,
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2021-06-10T00:00:00.000000+0000", options={}),
-            "1m",
+            "P1M",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-31T00:00:00.000000+0000"},
-                {"start_time": "2021-02-01T00:00:00.000000+0000", "end_time": "2021-02-28T00:00:00.000000+0000"},
-                {"start_time": "2021-03-01T00:00:00.000000+0000", "end_time": "2021-03-31T00:00:00.000000+0000"},
-                {"start_time": "2021-04-01T00:00:00.000000+0000", "end_time": "2021-04-30T00:00:00.000000+0000"},
-                {"start_time": "2021-05-01T00:00:00.000000+0000", "end_time": "2021-05-31T00:00:00.000000+0000"},
+                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-31T23:59:59.999999+0000"},
+                {"start_time": "2021-02-01T00:00:00.000000+0000", "end_time": "2021-02-28T23:59:59.999999+0000"},
+                {"start_time": "2021-03-01T00:00:00.000000+0000", "end_time": "2021-03-31T23:59:59.999999+0000"},
+                {"start_time": "2021-04-01T00:00:00.000000+0000", "end_time": "2021-04-30T23:59:59.999999+0000"},
+                {"start_time": "2021-05-01T00:00:00.000000+0000", "end_time": "2021-05-31T23:59:59.999999+0000"},
                 {"start_time": "2021-06-01T00:00:00.000000+0000", "end_time": "2021-06-10T00:00:00.000000+0000"},
             ],
         ),
@@ -115,12 +120,13 @@ def mock_datetime_now(monkeypatch):
             None,
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2022-06-10T00:00:00.000000+0000", options={}),
-            "1y",
+            "P1Y",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-12-31T00:00:00.000000+0000"},
+                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-12-31T23:59:59.999999+0000"},
                 {"start_time": "2022-01-01T00:00:00.000000+0000", "end_time": "2022-01-01T00:00:00.000000+0000"},
             ],
         ),
@@ -129,16 +135,17 @@ def mock_datetime_now(monkeypatch):
             {"date": "2021-01-05T00:00:00.000000+0000"},
             MinMaxDatetime(datetime="{{ stream_state['date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            "1d",
+            "P1D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
-                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T00:00:00.000000+0000"},
-                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T00:00:00.000000+0000"},
-                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T00:00:00.000000+0000"},
-                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T00:00:00.000000+0000"},
+                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T23:59:59.999999+0000"},
+                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T23:59:59.999999+0000"},
+                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T23:59:59.999999+0000"},
+                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T23:59:59.999999+0000"},
+                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T23:59:59.999999+0000"},
                 {"start_time": "2021-01-10T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
             ],
         ),
@@ -147,10 +154,11 @@ def mock_datetime_now(monkeypatch):
             None,
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            "12d",
+            "P12D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
                 {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
             ],
@@ -160,15 +168,16 @@ def mock_datetime_now(monkeypatch):
             None,
             MinMaxDatetime(datetime="2021-12-28T00:00:00.000000+0000", options={}),
             MinMaxDatetime(datetime=f"{(FAKE_NOW + datetime.timedelta(days=1)).strftime(datetime_format)}", options={}),
-            "1d",
+            "P1D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-12-28T00:00:00.000000+0000", "end_time": "2021-12-28T00:00:00.000000+0000"},
-                {"start_time": "2021-12-29T00:00:00.000000+0000", "end_time": "2021-12-29T00:00:00.000000+0000"},
-                {"start_time": "2021-12-30T00:00:00.000000+0000", "end_time": "2021-12-30T00:00:00.000000+0000"},
-                {"start_time": "2021-12-31T00:00:00.000000+0000", "end_time": "2021-12-31T00:00:00.000000+0000"},
+                {"start_time": "2021-12-28T00:00:00.000000+0000", "end_time": "2021-12-28T23:59:59.999999+0000"},
+                {"start_time": "2021-12-29T00:00:00.000000+0000", "end_time": "2021-12-29T23:59:59.999999+0000"},
+                {"start_time": "2021-12-30T00:00:00.000000+0000", "end_time": "2021-12-30T23:59:59.999999+0000"},
+                {"start_time": "2021-12-31T00:00:00.000000+0000", "end_time": "2021-12-31T23:59:59.999999+0000"},
                 {"start_time": "2022-01-01T00:00:00.000000+0000", "end_time": "2022-01-01T00:00:00.000000+0000"},
             ],
         ),
@@ -177,10 +186,11 @@ def mock_datetime_now(monkeypatch):
             None,
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
             MinMaxDatetime(datetime="2021-01-05T00:00:00.000000+0000", options={}),
-            "1d",
+            "P1D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
                 {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
             ],
@@ -190,16 +200,17 @@ def mock_datetime_now(monkeypatch):
             {"date": "2021-01-05T00:00:00.000000+0000"},
             MinMaxDatetime(datetime="{{ stream_state['date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            "1d",
+            "P1D",
             InterpolatedString(string="{{ stream_state['date'] }}", options={}),
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
-                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T00:00:00.000000+0000"},
-                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T00:00:00.000000+0000"},
-                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T00:00:00.000000+0000"},
-                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T00:00:00.000000+0000"},
+                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T23:59:59.999999+0000"},
+                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T23:59:59.999999+0000"},
+                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T23:59:59.999999+0000"},
+                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T23:59:59.999999+0000"},
+                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T23:59:59.999999+0000"},
                 {"start_time": "2021-01-10T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
             ],
         ),
@@ -208,14 +219,15 @@ def mock_datetime_now(monkeypatch):
             {cursor_field: "2021-01-05T00:00:00.000000+0000"},
             MinMaxDatetime(datetime="2021-01-03T00:00:00.000000+0000", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            "2d",
+            "P2D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-07T00:00:00.000000+0000"},
-                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-09T00:00:00.000000+0000"},
-                {"start_time": "2021-01-10T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
+                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-06T23:59:59.999999+0000"},
+                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-08T23:59:59.999999+0000"},
+                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
             ],
         ),
         (
@@ -223,16 +235,17 @@ def mock_datetime_now(monkeypatch):
             {"date": "2021-01-05T00:00:00.000000+0000"},
             MinMaxDatetime(datetime="{{ config['start_date'] }}", min_datetime="{{ stream_state['date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            "1d",
+            "P1D",
             InterpolatedString(string="{{ stream_state['date'] }}", options={}),
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
-                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T00:00:00.000000+0000"},
-                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T00:00:00.000000+0000"},
-                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T00:00:00.000000+0000"},
-                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T00:00:00.000000+0000"},
+                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T23:59:59.999999+0000"},
+                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T23:59:59.999999+0000"},
+                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T23:59:59.999999+0000"},
+                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T23:59:59.999999+0000"},
+                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T23:59:59.999999+0000"},
                 {"start_time": "2021-01-10T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
             ],
         ),
@@ -241,15 +254,16 @@ def mock_datetime_now(monkeypatch):
             {"date": "2021-01-05T00:00:00.000000+0000"},
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", max_datetime="{{ stream_state['date'] }}", options={}),
-            "1d",
+            "P1D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-01T00:00:00.000000+0000"},
-                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T00:00:00.000000+0000"},
-                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T00:00:00.000000+0000"},
-                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T00:00:00.000000+0000"},
+                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-01T23:59:59.999999+0000"},
+                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T23:59:59.999999+0000"},
+                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T23:59:59.999999+0000"},
+                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T23:59:59.999999+0000"},
                 {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
             ],
         ),
@@ -258,10 +272,11 @@ def mock_datetime_now(monkeypatch):
             {"date": "2021-01-05"},
             MinMaxDatetime(datetime="{{ config['start_date_ymd'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10", max_datetime="{{ stream_state['date'] }}", options={}),
-            "1d",
+            "P1D",
             cursor_field,
             None,
             "%Y-%m-%d",
+            "P1D",
             [
                 {"start_time": "2021-01-01", "end_time": "2021-01-01"},
                 {"start_time": "2021-01-02", "end_time": "2021-01-02"},
@@ -275,18 +290,19 @@ def mock_datetime_now(monkeypatch):
             {"date": "2021-01-05"},
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10", max_datetime="{{ stream_state['date'] }}", datetime_format="%Y-%m-%d", options={}),
-            "1d",
+            "P1D",
             cursor_field,
-            "3d",
+            "P3D",
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2020-12-29T00:00:00.000000+0000", "end_time": "2020-12-29T00:00:00.000000+0000"},
-                {"start_time": "2020-12-30T00:00:00.000000+0000", "end_time": "2020-12-30T00:00:00.000000+0000"},
-                {"start_time": "2020-12-31T00:00:00.000000+0000", "end_time": "2020-12-31T00:00:00.000000+0000"},
-                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-01T00:00:00.000000+0000"},
-                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T00:00:00.000000+0000"},
-                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T00:00:00.000000+0000"},
-                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T00:00:00.000000+0000"},
+                {"start_time": "2020-12-29T00:00:00.000000+0000", "end_time": "2020-12-29T23:59:59.999999+0000"},
+                {"start_time": "2020-12-30T00:00:00.000000+0000", "end_time": "2020-12-30T23:59:59.999999+0000"},
+                {"start_time": "2020-12-31T00:00:00.000000+0000", "end_time": "2020-12-31T23:59:59.999999+0000"},
+                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-01T23:59:59.999999+0000"},
+                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T23:59:59.999999+0000"},
+                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T23:59:59.999999+0000"},
+                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T23:59:59.999999+0000"},
                 {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
             ],
         ),
@@ -295,14 +311,16 @@ def mock_datetime_now(monkeypatch):
             {cursor_field: "2021-01-05T00:00:00.000000+0000"},
             MinMaxDatetime(datetime="2021-01-01T00:00:00.000000+0000", options={}),
             MinMaxDatetime(datetime="2021-01-06T00:00:00.000000+0000", options={}),
-            "1d",
+            "P1D",
             cursor_field,
-            "3d",
+            "P3D",
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T00:00:00.000000+0000"},
-                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T00:00:00.000000+0000"},
-                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
+                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T23:59:59.999999+0000"},
+                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T23:59:59.999999+0000"},
+                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T23:59:59.999999+0000"},
+                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T23:59:59.999999+0000"},
                 {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T00:00:00.000000+0000"},
             ],
         ),
@@ -311,15 +329,16 @@ def mock_datetime_now(monkeypatch):
             {"date": "2021-01-05"},
             MinMaxDatetime(datetime="{{ config['start_date'] }}", options={}),
             MinMaxDatetime(datetime="2021-01-10", max_datetime="{{ stream_state['date'] }}", datetime_format="%Y-%m-%d", options={}),
-            "1d",
+            "P1D",
             cursor_field,
             "{{ config['does_not_exist'] }}",
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-01T00:00:00.000000+0000"},
-                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T00:00:00.000000+0000"},
-                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T00:00:00.000000+0000"},
-                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T00:00:00.000000+0000"},
+                {"start_time": "2021-01-01T00:00:00.000000+0000", "end_time": "2021-01-01T23:59:59.999999+0000"},
+                {"start_time": "2021-01-02T00:00:00.000000+0000", "end_time": "2021-01-02T23:59:59.999999+0000"},
+                {"start_time": "2021-01-03T00:00:00.000000+0000", "end_time": "2021-01-03T23:59:59.999999+0000"},
+                {"start_time": "2021-01-04T00:00:00.000000+0000", "end_time": "2021-01-04T23:59:59.999999+0000"},
                 {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T00:00:00.000000+0000"},
             ],
         ),
@@ -328,22 +347,34 @@ def mock_datetime_now(monkeypatch):
             {cursor_field: "2021-01-05T00:00:00.000000+0000"},
             MinMaxDatetime(datetime="2021-01-01T00:00:00.000000+0000", options={}),
             MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            "1d",
+            "P1D",
             cursor_field,
             None,
             datetime_format,
+            cursor_granularity,
             [
-                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T00:00:00.000000+0000"},
-                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T00:00:00.000000+0000"},
-                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T00:00:00.000000+0000"},
-                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T00:00:00.000000+0000"},
+                {"start_time": "2021-01-05T00:00:00.000000+0000", "end_time": "2021-01-05T23:59:59.999999+0000"},
+                {"start_time": "2021-01-06T00:00:00.000000+0000", "end_time": "2021-01-06T23:59:59.999999+0000"},
+                {"start_time": "2021-01-07T00:00:00.000000+0000", "end_time": "2021-01-07T23:59:59.999999+0000"},
+                {"start_time": "2021-01-08T00:00:00.000000+0000", "end_time": "2021-01-08T23:59:59.999999+0000"},
+                {"start_time": "2021-01-09T00:00:00.000000+0000", "end_time": "2021-01-09T23:59:59.999999+0000"},
                 {"start_time": "2021-01-10T00:00:00.000000+0000", "end_time": "2021-01-10T00:00:00.000000+0000"},
             ],
         ),
     ],
 )
 def test_stream_slices(
-    mock_datetime_now, test_name, stream_state, start, end, step, cursor_field, lookback_window, datetime_format, expected_slices
+    mock_datetime_now,
+    test_name,
+    stream_state,
+    start,
+    end,
+    step,
+    cursor_field,
+    lookback_window,
+    datetime_format,
+    cursor_granularity,
+    expected_slices,
 ):
     lookback_window = InterpolatedString(string=lookback_window, options={}) if lookback_window else None
     slicer = DatetimeStreamSlicer(
@@ -352,13 +383,14 @@ def test_stream_slices(
         step=step,
         cursor_field=cursor_field,
         datetime_format=datetime_format,
+        cursor_granularity=cursor_granularity,
         lookback_window=lookback_window,
         config=config,
         options={},
     )
     stream_slices = slicer.stream_slices(SyncMode.incremental, stream_state)
 
-    assert expected_slices == stream_slices
+    assert stream_slices == expected_slices
 
 
 @pytest.mark.parametrize(
@@ -406,10 +438,11 @@ def test_update_cursor(test_name, previous_cursor, stream_slice, last_record, ex
     slicer = DatetimeStreamSlicer(
         start_datetime=MinMaxDatetime(datetime="2021-01-01T00:00:00.000000+0000", options={}),
         end_datetime=MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-        step="1d",
+        step="P1D",
         cursor_field=InterpolatedString(string=cursor_field, options={}),
         datetime_format=datetime_format,
-        lookback_window=InterpolatedString(string="0d", options={}),
+        cursor_granularity=cursor_granularity,
+        lookback_window=InterpolatedString(string="P0D", options={}),
         config=config,
         options={},
     )
@@ -478,10 +511,11 @@ def test_request_option(test_name, inject_into, field_name, expected_req_params,
             DatetimeStreamSlicer(
                 start_datetime=MinMaxDatetime(datetime="2021-01-01T00:00:00.000000+0000", options={}),
                 end_datetime=MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-                step="1d",
+                step="P1D",
                 cursor_field=InterpolatedString(string=cursor_field, options={}),
                 datetime_format=datetime_format,
-                lookback_window=InterpolatedString(string="0d", options={}),
+                cursor_granularity=cursor_granularity,
+                lookback_window=InterpolatedString(string="P0D", options={}),
                 start_time_option=start_request_option,
                 end_time_option=end_request_option,
                 config=config,
@@ -496,10 +530,11 @@ def test_request_option(test_name, inject_into, field_name, expected_req_params,
         slicer = DatetimeStreamSlicer(
             start_datetime=MinMaxDatetime(datetime="2021-01-01T00:00:00.000000+0000", options={}),
             end_datetime=MinMaxDatetime(datetime="2021-01-10T00:00:00.000000+0000", options={}),
-            step="1d",
+            step="P1D",
             cursor_field=InterpolatedString(string=cursor_field, options={}),
             datetime_format=datetime_format,
-            lookback_window=InterpolatedString(string="0d", options={}),
+            cursor_granularity=cursor_granularity,
+            lookback_window=InterpolatedString(string="P0D", options={}),
             start_time_option=start_request_option,
             end_time_option=end_request_option,
             config=config,
@@ -516,31 +551,34 @@ def test_request_option(test_name, inject_into, field_name, expected_req_params,
 
 
 @pytest.mark.parametrize(
-    "test_name, input_date, date_format, expected_output_date",
+    "test_name, input_date, date_format, date_format_granularity, expected_output_date",
     [
         (
             "test_parse_date_iso",
             "2021-01-01T00:00:00.000000+0000",
             "%Y-%m-%dT%H:%M:%S.%f%z",
+            "PT0.000001S",
             datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
         ),
         (
             "test_parse_timestamp",
             "1609459200",
             "%s",
+            "PT1S",
             datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
         ),
-        ("test_parse_date_number", "20210101", "%Y%m%d", datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc)),
+        ("test_parse_date_number", "20210101", "%Y%m%d", "P1D", datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc)),
     ],
 )
-def test_parse_date(test_name, input_date, date_format, expected_output_date):
+def test_parse_date(test_name, input_date, date_format, date_format_granularity, expected_output_date):
     slicer = DatetimeStreamSlicer(
         start_datetime=MinMaxDatetime("2021-01-01T00:00:00.000000+0000", options={}),
         end_datetime=MinMaxDatetime("2021-01-10T00:00:00.000000+0000", options={}),
-        step="1d",
+        step="P1D",
         cursor_field=InterpolatedString(cursor_field, options={}),
         datetime_format=date_format,
-        lookback_window=InterpolatedString("0d", options={}),
+        cursor_granularity=date_format_granularity,
+        lookback_window=InterpolatedString("P0D", options={}),
         config=config,
         options={},
     )
@@ -549,21 +587,22 @@ def test_parse_date(test_name, input_date, date_format, expected_output_date):
 
 
 @pytest.mark.parametrize(
-    "test_name, input_dt, datetimeformat, expected_output",
+    "test_name, input_dt, datetimeformat, datetimeformat_granularity, expected_output",
     [
-        ("test_format_timestamp", datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), "%s", "1609459200"),
-        ("test_format_string", datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), "%Y-%m-%d", "2021-01-01"),
-        ("test_format_to_number", datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), "%Y%m%d", "20210101"),
+        ("test_format_timestamp", datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), "%s", "PT1S", "1609459200"),
+        ("test_format_string", datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), "%Y-%m-%d", "P1D", "2021-01-01"),
+        ("test_format_to_number", datetime.datetime(2021, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), "%Y%m%d", "P1D", "20210101"),
     ],
 )
-def test_format_datetime(test_name, input_dt, datetimeformat, expected_output):
+def test_format_datetime(test_name, input_dt, datetimeformat, datetimeformat_granularity, expected_output):
     slicer = DatetimeStreamSlicer(
         start_datetime=MinMaxDatetime("2021-01-01T00:00:00.000000+0000", options={}),
         end_datetime=MinMaxDatetime("2021-01-10T00:00:00.000000+0000", options={}),
-        step="1d",
+        step="P1D",
         cursor_field=InterpolatedString(cursor_field, options={}),
         datetime_format=datetimeformat,
-        lookback_window=InterpolatedString("0d", options={}),
+        cursor_granularity=datetimeformat_granularity,
+        lookback_window=InterpolatedString("P0D", options={}),
         config=config,
         options={},
     )
